@@ -166,8 +166,6 @@ static void _app_setup_window (GtkWidget * w, Application *app) {
         GTK_SIGNAL_FUNC (cb_view_area_expose), app);
     g_signal_connect (G_OBJECT (w), "button-press-event",
         GTK_SIGNAL_FUNC (cb_view_area_button_press), app);
-    g_signal_connect (G_OBJECT (w), "configure-event",
-            GTK_SIGNAL_FUNC (cb_view_area_configure), app);
 }
 
 void app_full_screen (Application *app) {
@@ -189,16 +187,16 @@ void app_full_screen (Application *app) {
             g_signal_connect (G_OBJECT (win), "key-press-event",
                     GTK_SIGNAL_FUNC (cb_keyPressed), (void *)app);
         }
-        gtk_widget_show_all (GTK_WIDGET (app->window_fullscreen));
         app->fullscreen = true;
+        gtk_widget_show_all (GTK_WIDGET (app->window_fullscreen));
         app_update_screen_blanking (app);
     }
 }
 
 void app_normal_screen (Application *app) {
     if (app->fullscreen) {
-        gtk_widget_hide (GTK_WIDGET (app->window_fullscreen));
         app->fullscreen = false;
+        gtk_widget_hide (GTK_WIDGET (app->window_fullscreen));
     }
     app_update_screen_blanking (app);
 }
