@@ -1,5 +1,5 @@
 /**
- * Copyright (C) 2005 by Koos Vriezen <koos.vriezen@gmail.com>
+ * Copyright (C) 2005 by Koos Vriezen <koos ! vriezen ? xs4all ! nl>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -16,17 +16,21 @@
  * Boston, MA 02110-1301, USA.
  **/
 
-#include <config.h>
-#include <kdebug.h>
-#include <kurl.h>
-
+//#include <config.h>
+#include <string.h>
 #include "kmplayer_asx.h"
 
 using namespace KMPlayer;
 
+static String getAsxAttribute (Element * e, const String & attr) {
+    for (Attribute *a = e->attributes ()->first (); a; a = a->nextSibling ())
+        if (attr == a->name ().toString ().lower ())
+            return a->value ();
+    return String ();
+}
 
-KDE_NO_EXPORT NodePtr ASX::Asx::childFromTag (const QString & tag) {
-    const char * name = tag.latin1 ();
+KMPLAYER_NO_EXPORT Node *ASX::Asx::childFromTag (const String & tag) {
+    const char * name = (const char *) tag;
     if (!strcasecmp (name, "entry"))
         return new ASX::Entry (m_doc);
     else if (!strcasecmp (name, "entryref"))
@@ -37,24 +41,24 @@ KDE_NO_EXPORT NodePtr ASX::Asx::childFromTag (const QString & tag) {
         return new DarkNode (m_doc, name, id_node_base);
     else if (!strcasecmp (name, "param"))
         return new DarkNode (m_doc, name, id_node_param);
-    return 0L;
+    return NULL;
 }
 
 KDE_NO_EXPORT Node::PlayType ASX::Asx::playType () {
     if (cached_ismrl_version != document ()->m_tree_version)
-        for (NodePtr e = firstChild (); e; e = e->nextSibling ()) {
+        for (Node *e = firstChild (); e; e = e->nextSibling ()) {
             if (e->id == id_node_title)
-                pretty_name = e->innerText ().simplifyWhiteSpace ();
+                title = e->innerText ().stripWhiteSpace ();
             else if (e->id == id_node_base)
-                src = convertNode <Element> (e)->getAttribute ("href");
+                src = getAsxAttribute (static_cast <Element *> (e), "href");
         }
     return Mrl::playType ();
 }
 
 //-----------------------------------------------------------------------------
 
-KDE_NO_EXPORT NodePtr ASX::Entry::childFromTag (const QString & tag) {
-    const char * name = tag.latin1 ();
+KMPLAYER_NO_EXPORT Node *ASX::Entry::childFromTag (const String & tag) {
+    const char * name = (const char *) tag;
     if (!strcasecmp (name, "ref"))
         return new ASX::Ref (m_doc);
     else if (!strcasecmp (name, "title"))
@@ -67,73 +71,91 @@ KDE_NO_EXPORT NodePtr ASX::Entry::childFromTag (const QString & tag) {
         return new DarkNode (m_doc, name, id_node_starttime);
     else if (!strcasecmp (name, "duration"))
         return new DarkNode (m_doc, name, id_node_duration);
-    return 0L;
+    return NULL;
 }
 
 KDE_NO_EXPORT Node::PlayType ASX::Entry::playType () {
     if (cached_ismrl_version != document ()->m_tree_version) {
         ref_child_count = 0;
-        NodePtr ref;
-        for (NodePtr e = firstChild (); e; e = e->nextSibling ()) {
+        Node *ref = NULL;
+        for (Node *e = firstChild (); e; e = e->nextSibling ()) {
             switch (e->id) {
             case id_node_title:
-                pretty_name = e->innerText (); // already normalized (hopefully)
+                title = e->innerText ().stripWhiteSpace ();
                 break;
             case id_node_base:
-                src = convertNode <Element> (e)->getAttribute ("href");
+                src = getAsxAttribute (static_cast <Element *> (e), "href");
                 break;
             case id_node_ref:
                 ref = e;
                 ref_child_count++;
             }
         }
-        if (ref_child_count == 1 && !pretty_name.isEmpty ())
-            convertNode <ASX::Ref> (ref)->pretty_name = pretty_name;
+        if (ref_child_count == 1 && !title.isEmpty ())
+            static_cast <ASX::Ref *> (ref)->title = title;
         cached_ismrl_version = document()->m_tree_version;
     }
     return play_type_none;
 }
 
-KDE_NO_EXPORT void ASX::Entry::activate () {
+KMPLAYER_NO_EXPORT void ASX::Entry::activate () {
     resolved = true;
-    for (NodePtr e = firstChild (); e; e = e->nextSibling ())
+    for (Node *e = firstChild (); e; e = e->nextSibling ())
         if (e->id == id_node_param) {
-            Element * elm = convertNode <Element> (e);
-            if (elm->getAttribute ("name").lower() == QString ("clipsummary")) {
-                PlayListNotify * n = document ()->notify_listener;
-                if (n)
-                    n->setInfoMessage (KURL::decode_string (elm->getAttribute ("value")));
-                break;
+            Element *elm = static_cast <Element *> (e);
+            if (getAsxAttribute(elm,"name").lower() == String("clipsummary")) {
+                String inf = URL::decode_string (
+                                getAsxAttribute (elm, "value"));
+                document ()->message (MsgInfoString, &inf);
             }
+        } else if (e->id == id_node_duration) {
+            String s = static_cast <Element *> (e)->getAttribute (
+                    StringPool::attr_value);
+            int pos = parseTimeString( s );
+            if (pos > 0)
+                duration_timer = document()->post (
+                        new TimerPosting (this, pos * 10));
         }
     Mrl::activate ();
 }
 
-KDE_NO_EXPORT void ASX::Entry::deactivate () {
-    PlayListNotify * n = document ()->notify_listener;
-    if (n)
-        n->setInfoMessage (QString ());
+KDE_NO_EXPORT void ASX::Entry::message (MessageType msg, void *content) {
+    if (msg == MsgEventTimer) {
+        duration_timer = NULL;
+        deactivate ();
+        return;
+    }
+    Mrl::message (msg, content);
 }
 
-KDE_NO_EXPORT bool ASX::Entry::expose () const {
-    return ref_child_count > 1 && !pretty_name.isEmpty ();
+KMPLAYER_NO_EXPORT void ASX::Entry::deactivate () {
+    document ()->message (MsgInfoString, NULL);
+    if (duration_timer) {
+        document()->cancelPosting (duration_timer);
+        duration_timer = NULL;
+    }
+    Mrl::deactivate ();
+}
+
+KMPLAYER_NO_EXPORT bool ASX::Entry::expose () const {
+    return ref_child_count > 1 && !title.isEmpty ();
 }
 
 //-----------------------------------------------------------------------------
 
-KDE_NO_EXPORT void ASX::Ref::opened () {
-    src = getAttribute ("href");
-    //kdDebug () << "Ref attr found src: " << src << endl;
+KMPLAYER_NO_EXPORT void ASX::Ref::opened () {
+    src = getAsxAttribute (this, "href");
+    Mrl::opened ();
 }
 
-KDE_NO_EXPORT bool ASX::Ref::expose () const {
+KMPLAYER_NO_EXPORT bool ASX::Ref::expose () const {
     return !src.isEmpty ();
 }
 
 //-----------------------------------------------------------------------------
 
-KDE_NO_EXPORT void ASX::EntryRef::opened () {
-    src = getAttribute ("href");
-    //kdDebug () << "EntryRef attr found src: " << src << endl;
+KMPLAYER_NO_EXPORT void ASX::EntryRef::opened () {
+    src = getAsxAttribute (this, "href");
+    Mrl::opened ();
 }
 
