@@ -590,9 +590,16 @@ void Generator::readEvent (UnixProcess *) {
 void Generator::writeEvent (UnixProcess *) {
     if (media_info) {
         ByteArray &ba = media_info->rawData ();
-        int nr = unix_process->writeStdIn (
-                ba.data() + write_pos, ba.size () - write_pos);
+        const gchar *start = ba.data () + write_pos;
+        const gchar *end;
+        int wmax = ba.size () - write_pos;
+        int wcount = wmax;
+        if (!g_utf8_validate (start, wmax, &end))
+            wcount = end - start;
+        int nr = unix_process->writeStdIn (start, wcount);
         write_pos += nr;
+        if (nr == wcount && wcount < wmax)
+            write_pos++; //skip
         if (write_pos < ba.size ())
             return;
     }
