@@ -1,5 +1,5 @@
 /**
- * Copyright (C) 2006 by Koos Vriezen <koos.vriezen@gmail.com>
+ * Copyright (C) 2006-2007 by Koos Vriezen <koos.vriezen@gmail.com>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -18,14 +18,11 @@
 
 #include <config.h>
 
-#include <qcolor.h>
-#include <qimage.h>
-#include <qtimer.h>
-
-#include <kdebug.h>
+#include <string.h>
 
 #include "kmplayer_rp.h"
 #include "kmplayer_smil.h"
+#include "actor.h"
 
 using namespace KMPlayer;
 
@@ -34,28 +31,43 @@ KDE_NO_CDTOR_EXPORT RP::Imfl::Imfl (NodePtr & d)
   : Mrl (d, id_node_imfl),
     fit (fit_hidden),
     duration (0),
+    duration_timer (NULL),
     needs_scene_img (0) {}
 
 KDE_NO_CDTOR_EXPORT RP::Imfl::~Imfl () {
 }
 
+KDE_NO_EXPORT void RP::Imfl::closed () {
+    for (Node *n = firstChild (); n; n = n->nextSibling ())
+        if (RP::id_node_head == n->id) {
+            Attribute *a = static_cast <Element *> (n)->attributes ()->first ();
+            for (; a; a = a->nextSibling ()) {
+                if (StringPool::attr_width == a->name ()) {
+                    size.width = a->value ().toInt ();
+                } else if (StringPool::attr_height == a->name ()) {
+                    size.height = a->value ().toInt ();
+                } else if (a->name () == "duration") {
+                    int dur;
+                    parseTime (a->value ().lower (), dur);
+                    duration = dur;
+                }
+            }
+        }
+    Mrl::closed ();
+}
+
 KDE_NO_EXPORT void RP::Imfl::defer () {
-    kdDebug () << "RP::Imfl::defer " << endl;
+    debugLog () << "RP::Imfl::defer " << endl;
     setState (state_deferred);
-    for (Node * n = firstChild ().ptr (); n; n = n->nextSibling ().ptr ())
+    for (Node *n = firstChild (); n; n = n->nextSibling ())
         if (n->id == RP::id_node_image && !n->active ())
             n->activate ();
 }
 
 KDE_NO_EXPORT void RP::Imfl::activate () {
-    kdDebug () << "RP::Imfl::activate " << endl;
+    debugLog () << "RP::Imfl::activate " << endl;
     resolved = true;
     setState (state_activated);
-    surface = Mrl::getSurface (this);
-    if (!surface) {
-        finish ();
-        return;
-    }
     int timings_count = 0;
     for (NodePtr n = firstChild (); n; n = n->nextSibling ())
         switch (n->id) {
@@ -71,39 +83,20 @@ KDE_NO_EXPORT void RP::Imfl::activate () {
             case RP::id_node_image:
                 if (!n->active ())
                     n->activate ();
-            case RP::id_node_head:
-                for (AttributePtr a= convertNode <Element> (n)->attributes ()->first (); a; a = a->nextSibling ()) {
-                    if (!strcmp (a->nodeName (), "width")) {
-                        width = a->nodeValue ().toInt ();
-                    } else if (!strcmp (a->nodeName (), "height")) {
-                        height = a->nodeValue ().toInt ();
-                    } else if (!strcmp (a->nodeName (), "duration")) {
-                        int dur;
-                        parseTime (a->nodeValue ().lower (), dur);
-                        duration = dur;
-                    }
-                }
                 break;
         }
-    if (width <= 0 || width > 32000)
-        width = surface->bounds.width ();
-    if (height <= 0 || height > 32000)
-        height = surface->bounds.height ();
-    if (width > 0 && height > 0) {
-        surface->xscale = 1.0 * surface->bounds.width () / width;
-        surface->yscale = 1.0 * surface->bounds.height () / height;
-    }
     if (duration > 0)
-        duration_timer = document ()->setTimeout (this, duration * 100);
+        duration_timer = document ()->post (
+                new TimerPosting (this, duration * 10));
     else if (!timings_count)
         finish ();
 }
 
 KDE_NO_EXPORT void RP::Imfl::finish () {
-    kdDebug () << "RP::Imfl::finish " << endl;
+    debugLog () << "RP::Imfl::finish " << endl;
     Mrl::finish ();
     if (duration_timer) {
-        document ()->cancelTimer (duration_timer);
+        document ()->cancelPosting (duration_timer);
         duration_timer = 0;
     }
     for (NodePtr n = firstChild (); n; n = n->nextSibling ())
@@ -111,66 +104,65 @@ KDE_NO_EXPORT void RP::Imfl::finish () {
             n->finish ();
 }
 
-KDE_NO_EXPORT void RP::Imfl::childDone (NodePtr) {
-    if (unfinished () && !duration_timer) {
-        for (NodePtr n = firstChild (); n; n = n->nextSibling ())
-            switch (n->id) {
-                case RP::id_node_crossfade:
-                case RP::id_node_fadein:
-                case RP::id_node_fadeout:
-                case RP::id_node_fill:
-                    if (n->unfinished ())
-                        return;
-            }
-        finish ();
-    }
-}
-
 KDE_NO_EXPORT void RP::Imfl::deactivate () {
-    kdDebug () << "RP::Imfl::deactivate " << endl;
+    //debugLog () << "RP::Imfl::deactivate " << endl;
     if (unfinished ())
         finish ();
+    else if (duration_timer) {
+        document ()->cancelPosting (duration_timer);
+        duration_timer = 0;
+    }
     if (!active ())
         return; // calling finish might call deactivate() as well
     setState (state_deactivated);
     for (NodePtr n = firstChild (); n; n = n->nextSibling ())
         if (n->active ())
             n->deactivate ();
-    surface = Mrl::getSurface (0L);
+    rp_surface = (Surface *) role (RoleChildDisplay, NULL);
 }
 
-KDE_NO_EXPORT bool RP::Imfl::handleEvent (EventPtr event) {
-    if (event->id () == event_sized) {
-        fit = static_cast <SizeEvent *> (event.ptr ())->fit;
-        if (surface) {
-            if (fit == fit_fill) {
-                surface->xscale = 1.0 * surface->bounds.width () / width;
-                surface->yscale = 1.0 * surface->bounds.height () / height;
-            } else { // keep aspect, center (and so add offset) ??
-                if (surface->xscale > surface->yscale)
-                    surface->xscale = surface->yscale;
-                else
-                    surface->yscale = surface->xscale;
-            }
-        }
-    } else if (event->id () == event_timer) {
-        TimerEvent * te = static_cast <TimerEvent *> (event.ptr ());
-        if (te->timer_info == duration_timer) {
-            kdDebug () << "RP::Imfl timer " << duration << endl;
+KDE_NO_EXPORT void RP::Imfl::message (MessageType msg, void *content) {
+    switch (msg) {
+        case MsgEventTimer:
             duration_timer = 0;
             if (unfinished ())
                 finish ();
-        }
+            return;
+        case MsgChildFinished:
+            if (unfinished () && !duration_timer) {
+                for (Node *n = firstChild (); n; n = n->nextSibling ())
+                    switch (n->id) {
+                        case RP::id_node_crossfade:
+                        case RP::id_node_fadein:
+                        case RP::id_node_fadeout:
+                        case RP::id_node_fill:
+                            if (n->unfinished ())
+                                return;
+                    }
+                finish ();
+            }
+            return;
+        default:
+            break;
     }
-    return true;
+    Mrl::message (msg, content);
 }
 
 KDE_NO_EXPORT void RP::Imfl::accept (Visitor * v) {
     v->visit (this);
 }
 
-KDE_NO_EXPORT NodePtr RP::Imfl::childFromTag (const QString & tag) {
-    const char * ctag = tag.latin1 ();
+KDE_NO_EXPORT Surface *RP::Imfl::surface () {
+    if (!rp_surface) {
+        rp_surface = (Surface *) Mrl::role (RoleChildDisplay, this);
+        if (rp_surface && size.isEmpty ())
+            size = rp_surface->bounds.size;
+    }
+    return rp_surface.ptr ();
+}
+
+KDE_NO_EXPORT Node *RP::Imfl::childFromTag (const String & tag) {
+    const char * ctag = (const char *) tag;
     if (!strcmp (ctag, "head"))
         return new DarkNode (m_doc, "head", RP::id_node_head);
     else if (!strcmp (ctag, "image"))
@@ -187,132 +179,178 @@ KDE_NO_EXPORT NodePtr RP::Imfl::childFromTag (const QString & tag) {
         return new RP::Fadein (m_doc);
     else if (!strcmp (ctag, "fadeout"))
         return new RP::Fadeout (m_doc);
-    return 0L;
+    return NULL;
 }
 
 KDE_NO_EXPORT void RP::Imfl::repaint () {
-    if (!active ())
-        kdWarning () << "Spurious Imfl repaint" << endl;
-    else if (surface && width > 0 && height > 0)
-        surface->repaint (0, 0, width, height);
+    if (!active ()) {
+        warningLog () << "Spurious Imfl repaint" << endl;
+    } else if (surface () && !size.isEmpty ()) {
+        rp_surface->markDirty ();
+        rp_surface->repaint (SRect (0, 0, size));
+    }
 }
 
 KDE_NO_CDTOR_EXPORT RP::Image::Image (NodePtr & doc)
- : Mrl (doc, id_node_image)
-{}
+ : Mrl (doc, id_node_image) {
+    view_mode = WindowMode;
+}
 
 KDE_NO_CDTOR_EXPORT RP::Image::~Image () {
 }
 
 KDE_NO_EXPORT void RP::Image::closed () {
-    src = getAttribute ("name");
+    src = getAttribute (StringPool::attr_name);
+    Mrl::closed ();
 }
 
 KDE_NO_EXPORT void RP::Image::activate () {
-    kdDebug () << "RP::Image::activate" << endl;
     setState (state_activated);
     isPlayable (); // update src attribute
-    cached_img.setUrl (absolutePath ());
-    if (cached_img.data->isEmpty ())
-        wget (absolutePath ());
+    if (!media_info)
+        media_info = new MediaInfo (this, ActorAgent::Image);
+    media_info->wget (absolutePath ());
+}
+
+KDE_NO_EXPORT void RP::Image::begin () {
+    Node::begin ();
 }
 
 KDE_NO_EXPORT void RP::Image::deactivate () {
-    cached_img.setUrl (QString ());
+    if (img_surface) {
+        img_surface->remove ();
+        img_surface = NULL;
+    }
     setState (state_deactivated);
     postpone_lock = 0L;
+    if (media_info) {
+        delete media_info;
+        media_info = NULL;
+    }
 }
 
+KDE_NO_EXPORT void RP::Image::message (MessageType msg, void *content) {
+    if (msg == MsgMediaReady) {
+        if (media_info)
+            dataArrived ();
+    } else {
+        Mrl::message (msg, content);
+    }
+}
 
-KDE_NO_EXPORT void RP::Image::remoteReady (QByteArray & data) {
-    kdDebug () << "RP::Image::remoteReady" << endl;
-    if (!data.isEmpty () && cached_img.data->isEmpty ()) {
-        QImage * img = new QImage (data);
-        if (!img->isNull ())
-            cached_img.data->image = img;
-        else
-            delete img;
+KDE_NO_EXPORT void RP::Image::dataArrived () {
+    ImageActor *im = media_info->media ? (ImageActor *)media_info->media : NULL;
+    if (!im->isEmpty ()) {
+        size.width = im->cached_img->width;
+        size.height = im->cached_img->height;
     }
     postpone_lock = 0L;
 }
 
 KDE_NO_EXPORT bool RP::Image::isReady (bool postpone_if_not) {
-    if (downloading () && postpone_if_not)
+    if (media_info->downloading () && postpone_if_not)
         postpone_lock = document ()->postpone ();
-    return !downloading ();
+    return !media_info->downloading ();
+}
+
+KDE_NO_EXPORT Surface *RP::Image::surface () {
+    ImageActor *im = media_info && media_info->media
+        ? (ImageActor *)media_info->media : NULL;
+    if (im && !img_surface && !im->isEmpty ()) {
+        Node *p = parentNode ();
+        if (p && p->id == RP::id_node_imfl) {
+            Surface *ps = static_cast <RP::Imfl *> (p)->surface ();
+            if (ps)
+                img_surface = ps->createSurface (this,
+                        SRect (0, 0, size));
+        }
+    }
+    return img_surface;
 }
 
 KDE_NO_CDTOR_EXPORT RP::TimingsBase::TimingsBase (NodePtr & d, const short i)
- : Element (d, i), x (0), y (0), w (0), h (0), start (0), duration (0) {}
+ : Element (d, i), x (0), y (0), w (0), h (0), start (0), duration (0),
+   start_timer (NULL), duration_timer (NULL), update_timer (NULL) {}
 
 KDE_NO_EXPORT void RP::TimingsBase::activate () {
     setState (state_activated);
     x = y = w = h = 0;
     srcx = srcy = srcw = srch = 0;
-    for (Attribute * a= attributes ()->first ().ptr (); a; a = a->nextSibling ().ptr ()) {
-        if (!strcasecmp (a->nodeName (), "start")) {
+    for (Attribute *a = attributes()->first (); a; a = a->nextSibling ()) {
+        if (a->name () == StringPool::attr_target) {
+            for (Node *n = parentNode()->firstChild(); n; n= n->nextSibling()) {
+                if (n->isElementNode () &&
+                        static_cast <Element *> (n)->
+                            getAttribute ("handle") == a->value ())
+                    target = n->self ();
+            }
+        } else if (a->name () == "start") {
             int dur;
-            parseTime (a->nodeValue ().lower (), dur);
+            parseTime (a->value ().lower (), dur);
             start = dur;
-        } else if (!strcasecmp (a->nodeName (), "duration")) {
+        } else if (a->name () == "duration") {
             int dur;
-            parseTime (a->nodeValue ().lower (), dur);
+            parseTime (a->value ().lower (), dur);
             duration = dur;
-        } else if (!strcasecmp (a->nodeName (), "target")) {
-            for (NodePtr n = parentNode()->firstChild(); n; n= n->nextSibling())
-                if (convertNode <Element> (n)->getAttribute ("handle") == a->nodeValue ())
-                    target = n;
-        } else if (!strcasecmp (a->nodeName (), "dstx")) {
-            x = a->nodeValue ().toInt ();
-        } else if (!strcasecmp (a->nodeName (), "dsty")) {
-            y = a->nodeValue ().toInt ();
-        } else if (!strcasecmp (a->nodeName (), "dstw")) {
-            w = a->nodeValue ().toInt ();
-        } else if (!strcasecmp (a->nodeName (), "dsth")) {
-            h = a->nodeValue ().toInt ();
-        } else if (!strcasecmp (a->nodeName (), "srcx")) {
-            srcx = a->nodeValue ().toInt ();
-        } else if (!strcasecmp (a->nodeName (), "srcy")) {
-            srcy = a->nodeValue ().toInt ();
-        } else if (!strcasecmp (a->nodeName (), "srcw")) {
-            srcw = a->nodeValue ().toInt ();
-        } else if (!strcasecmp (a->nodeName (), "srch")) {
-            srch = a->nodeValue ().toInt ();
+        } else if (a->name () == "dstx") {
+            x = a->value ().toInt ();
+        } else if (a->name () == "dsty") {
+            y = a->value ().toInt ();
+        } else if (a->name () == "dstw") {
+            w = a->value ().toInt ();
+        } else if (a->name () == "dsth") {
+            h = a->value ().toInt ();
+        } else if (a->name () == "srcx") {
+            srcx = a->value ().toInt ();
+        } else if (a->name () == "srcy") {
+            srcy = a->value ().toInt ();
+        } else if (a->name () == "srcw") {
+            srcw = a->value ().toInt ();
+        } else if (a->name () == "srch") {
+            srch = a->value ().toInt ();
         }
     }
-    start_timer = document ()->setTimeout (this, start *100);
+    start_timer = document ()->post (new TimerPosting (this, start *10));
 }
 
 KDE_NO_EXPORT void RP::TimingsBase::deactivate () {
     if (unfinished ())
         finish ();
+    else
+        cancelTimers ();
     setState (state_deactivated);
 }
 
-KDE_NO_EXPORT bool RP::TimingsBase::handleEvent (EventPtr event) {
-    if (event->id () == event_timer) {
-        TimerEvent * te = static_cast <TimerEvent *> (event.ptr ());
-        if (te->timer_info == update_timer && duration > 0) {
-            update (100 * ++curr_step / duration);
-            te->interval = true;
-        } else if (te->timer_info == start_timer) {
-            start_timer = 0;
-            duration_timer = document ()->setTimeout (this, duration * 100);
-            begin ();
-        } else if (te->timer_info == duration_timer) {
-            duration_timer = 0;
-            update (100);
-            finish ();
-        } else
-            return false;
-        return true;
-    } else if (event->id () == event_postponed) {
-        if (!static_cast <PostponedEvent *> (event.ptr ())->is_postponed) {
-            document_postponed = 0L; // disconnect
-            update (duration > 0 ? 0 : 100);
+KDE_NO_EXPORT void RP::TimingsBase::message (MessageType msg, void *content) {
+    switch (msg) {
+        case MsgEventTimer: {
+            TimerPosting *te = static_cast <TimerPosting *> (content);
+            if (te == update_timer && duration > 0) {
+                update (200 * 10 * ++curr_step / duration);
+                te->interval = true;
+            } else if (te == start_timer) {
+                start_timer = 0;
+                duration_timer = document()->post (
+                        new TimerPosting (this, duration * 10));
+                begin ();
+            } else if (te == duration_timer) {
+                duration_timer = 0;
+                update (100);
+                finish ();
+            }
+            break;
         }
+        case MsgEventPostponed: {
+            PostponedEvent *pe = static_cast <PostponedEvent *> (content);
+            if (!pe->is_postponed) {
+                document_postponed.disconnect ();
+                update (duration > 0 ? 0 : 100);
+            }
+            break;
+        }
+        default:
+            Element::message (msg, content);
     }
-    return false;
 }
 
 KDE_NO_EXPORT void RP::TimingsBase::begin () {
@@ -321,34 +359,38 @@ KDE_NO_EXPORT void RP::TimingsBase::begin () {
     if (target)
         target->begin ();
     if (duration > 0) {
-        steps = duration; // 10/s updates
-        update_timer = document ()->setTimeout (this, 100); // 50ms
+        steps = duration / 20; // 5/s updates
+        update_timer = document ()->post (new TimerPosting (this, 200)); // 200ms
         curr_step = 1;
     }
 }
 
 KDE_NO_EXPORT void RP::TimingsBase::update (int percentage) {
     progress = percentage;
-    Node * p = parentNode ().ptr ();
+    Node *p = parentNode ();
     if (p->id == RP::id_node_imfl)
         static_cast <RP::Imfl *> (p)->repaint ();
 }
 
 KDE_NO_EXPORT void RP::TimingsBase::finish () {
     progress = 100;
+    cancelTimers ();
+    document_postponed.disconnect ();
+    Element::finish ();
+}
+
+KDE_NO_EXPORT void RP::TimingsBase::cancelTimers () {
     if (start_timer) {
-        document ()->cancelTimer (start_timer);
+        document ()->cancelPosting (start_timer);
         start_timer = 0;
     } else if (duration_timer) {
-        document ()->cancelTimer (duration_timer);
+        document ()->cancelPosting (duration_timer);
         duration_timer = 0;
     }
     if (update_timer) {
-        document ()->cancelTimer (update_timer);
+        document ()->cancelPosting (update_timer);
         update_timer = 0;
     }
-    document_postponed = 0L; // disconnect
-    Element::finish ();
 }
 
 KDE_NO_EXPORT void RP::Crossfade::activate () {
@@ -356,12 +398,12 @@ KDE_NO_EXPORT void RP::Crossfade::activate () {
 }
 
 KDE_NO_EXPORT void RP::Crossfade::begin () {
-    //kdDebug () << "RP::Crossfade::begin" << endl;
+    //debugLog () << "RP::Crossfade::begin" << endl;
     TimingsBase::begin ();
     if (target && target->id == id_node_image) {
         RP::Image * img = static_cast <RP::Image *> (target.ptr ());
         if (!img->isReady (true))
-            document_postponed = document()->connectTo (this, event_postponed);
+            document_postponed.connect (document(), MsgEventPostponed, this);
         else
             update (duration > 0 ? 0 : 100);
     }
@@ -378,12 +420,12 @@ KDE_NO_EXPORT void RP::Fadein::activate () {
 }
 
 KDE_NO_EXPORT void RP::Fadein::begin () {
-    //kdDebug () << "RP::Fadein::begin" << endl;
+    //debugLog () << "RP::Fadein::begin" << endl;
     TimingsBase::begin ();
     if (target && target->id == id_node_image) {
         RP::Image * img = static_cast <RP::Image *> (target.ptr ());
         if (!img->isReady (true))
-            document_postponed = document()->connectTo (this, event_postponed);
+            document_postponed.connect (document(), MsgEventPostponed, this);
         else
             update (duration > 0 ? 0 : 100);
     }
@@ -394,12 +436,12 @@ KDE_NO_EXPORT void RP::Fadein::accept (Visitor * v) {
 }
 
 KDE_NO_EXPORT void RP::Fadeout::activate () {
-    to_color = QColor (getAttribute ("color")).rgb ();
+    to_color = Color (getAttribute ("color")).rgb ();
     TimingsBase::activate ();
 }
 
 KDE_NO_EXPORT void RP::Fadeout::begin () {
-    //kdDebug () << "RP::Fadeout::begin" << endl;
+    //debugLog () << "RP::Fadeout::begin" << endl;
     TimingsBase::begin ();
 }
 
@@ -408,7 +450,7 @@ KDE_NO_EXPORT void RP::Fadeout::accept (Visitor * v) {
 }
 
 KDE_NO_EXPORT void RP::Fill::activate () {
-    color = QColor (getAttribute ("color")).rgb ();
+    color = Color (getAttribute ("color")).rgb ();
     TimingsBase::activate ();
 }
 
@@ -423,24 +465,24 @@ KDE_NO_EXPORT void RP::Fill::accept (Visitor * v) {
 
 KDE_NO_EXPORT void RP::Wipe::activate () {
     //TODO implement 'type="push"'
-    QString dir = getAttribute ("direction").lower ();
+    String dir = getAttribute ("direction").lower ();
     direction = dir_right;
-    if (dir == QString::fromLatin1 ("left"))
+    if (dir == "left")
         direction = dir_left;
-    else if (dir == QString::fromLatin1 ("up"))
+    else if (dir == "up")
         direction = dir_up;
-    else if (dir == QString::fromLatin1 ("down"))
+    else if (dir == "down")
         direction = dir_down;
     TimingsBase::activate ();
 }
 
 KDE_NO_EXPORT void RP::Wipe::begin () {
-    //kdDebug () << "RP::Wipe::begin" << endl;
+    //debugLog () << "RP::Wipe::begin" << endl;
     TimingsBase::begin ();
     if (target && target->id == id_node_image) {
         RP::Image * img = static_cast <RP::Image *> (target.ptr ());
         if (!img->isReady (true))
-            document_postponed = document()->connectTo (this, event_postponed);
+            document_postponed.connect (document(), MsgEventPostponed, this);
         else
             update (duration > 0 ? 0 : 100);
     }
@@ -455,16 +497,16 @@ KDE_NO_EXPORT void RP::ViewChange::activate () {
 }
 
 KDE_NO_EXPORT void RP::ViewChange::begin () {
-    kdDebug () << "RP::ViewChange::begin" << endl;
+    //debugLog () << "RP::ViewChange::begin" << endl;
     setState (state_began);
-    Node * p = parentNode ().ptr ();
+    Node *p = parentNode ();
     if (p->id == RP::id_node_imfl)
         static_cast <RP::Imfl *> (p)->needs_scene_img++;
     update (0);
 }
 
 KDE_NO_EXPORT void RP::ViewChange::finish () {
-    Node * p = parentNode ().ptr ();
+    Node *p = parentNode ();
     if (p && p->id == RP::id_node_imfl)
         static_cast <RP::Imfl *> (p)->needs_scene_img--;
     TimingsBase::finish ();
