@@ -67,26 +67,36 @@ static gboolean playCurrentSchedule (void * ctr) {
 SourceDocument::SourceDocument (Control *c, const String &url)
     : Document (url, c), m_control (c) {}
 
-void *SourceDocument::message (MessageType msg, void *data) {
+void SourceDocument::message (MessageType msg, void *data) {
     switch (msg) {
 
-    case MsgQueryActorAgent:
-        return m_control->actorAgent ();
-
-    case MsgQueryRoleChildDisplay:
-        return VIEWAREA(m_control->m_app)->getSurface ((Mrl *) data);
-
     case MsgInfoString:
         app_set_info_content (m_control->m_app,
                 data ? (const char *) *((String *) data) : "", false);
-        return NULL;
+        return;
 
     default:
         break;
     }
-    return Document::message (msg, data);
+    Document::message (msg, data);
 }
 
+void *SourceDocument::role (RoleType msg, void *data) {
+    switch (msg) {
+
+    case RoleActorAgent:
+        return m_control->actorAgent ();
+
+    case RoleChildDisplay:
+        return VIEWAREA(m_control->m_app)->getSurface ((Mrl *) data);
+
+    default:
+        break;
+    }
+    return Document::role (msg, data);
+}
+
+
 Control::Control (Application * a)
     : m_process (0L), m_doc_timer (0), progress_banner (NULL),
       playlist_data (0L), m_config (new Config), m_app (a),
@@ -482,20 +492,20 @@ ActorAgent *Control::actorAgent () const {
  * --dest=com.nokia.osso_browser com.nokia.osso_browser.load_url \
  *  string:http://maemo.org/
  */
-bool Control::requestPlayURL (NodePtr mrl) {
-    debugLog () << "requestPlayURL " << mrl->mrl()->src << (mrl->mrl ()->view_mode == Mrl::SingleMode) << endl;
+bool Control::requestPlayURL (Mrl *mrl) {
+    debugLog () << "requestPlayURL " << mrl->src << (mrl->view_mode == Mrl::SingleMode) << endl;
     if (process ()->state () > Process::Ready) {
-        if (process ()->mrl () == mrl->mrl ()->linkNode ())
+        if (process ()->mrl () == mrl->linkNode ())
             return true;
         back_request = mrl; // still playing, schedule it
-        if (!setProcess (mrl->mrl ()))
+        if (!setProcess (mrl))
             process ()->stop ();
     } else {
-        if (mrl->mrl ()->view_mode == Mrl::SingleMode)
+        if (mrl->view_mode == Mrl::SingleMode)
             setCurrent (mrl);
         else
             back_request = mrl;
-        setProcess (mrl->mrl ());
+        setProcess (mrl);
         g_timeout_add (0, playCurrentSchedule, static_cast <void *> (this));
     }
     return true;
@@ -623,7 +633,7 @@ bool Control::setCurrent (NodePtr n) {
     return true;
 }
 
-static char * stateDesc [] = {
+static const char *stateDesc [] = {
     "init", "deferred", "activated", "began", "finished",
     "deactivated", "resetting"
 };
@@ -828,7 +838,6 @@ void Control::addRecent (const String &url, const String &title) {
             recents->appendChild (more);
         }
         if (more) {
-            KMPlayer::NodePtr item;
             if (count > 10) {
                 NodePtr item = more->previousSibling ();
                 recents->removeChild (item);
@@ -980,9 +989,11 @@ static Mrl *findFirstPlayable (Node *n) {
 }
 
 void KMPlayer::jump (Node *n, Control *ctr) {
-    Mrl *mrl = findFirstPlayable (n);
+    Mrl *mrl = n->document() != ctr->playlist_data->document
+        ? isListItem (n->id) ? n->mrl () : NULL
+        : findFirstPlayable (n);
     if (mrl && mrl->view_mode == Mrl::SingleMode) {
-        if (!mrl->isPlayable ())
+        if (n->document() == ctr->playlist_data->document && !mrl->isPlayable())
             return;
         if (ctr->process ()->playing () &&
                 ctr->process ()->mrl() == mrl->linkNode ()) {
@@ -993,7 +1004,7 @@ void KMPlayer::jump (Node *n, Control *ctr) {
         } else {
             //debugLog() << "jump " << mrl->src << endl;
             // FIXME: group activation doesn't work
-            ctr->back_request = id_node_group_node == n->id ? mrl : n;
+            ctr->back_request = mrl;
             if (ctr->process ()->playing ())
                 ctr->process ()->stop ();
             else
@@ -1051,7 +1062,7 @@ void cb_playlist_stop (GtkButton * button, void *p) {
 void cb_favorites_add (Application *app) {
     Control *ctr = CONTROL(app);
     bool valid = false;
-    gchar *msg = "Unknown failure";
+    const gchar *msg = "Unknown failure";
     if (!ctr->current_selected) {
         msg = "No current selected";
     } else {
