@@ -44,20 +44,24 @@ void Matrix::getWH (Single &w, Single &h) const {
     h *= d;
 }
 
-void Matrix::getXYWH (Single & x, Single & y, Single & w, Single & h) const {
-    getXY (x, y);
-    w *= a;
-    h *= d;
+IRect Matrix::toScreen (const SRect &rect) const {
+    return IRect (
+            (int) (Single (rect.x () * a) + tx),
+            (int) (Single (rect.y () * d) + ty),
+            (int) (rect.width () * a),
+            (int) (rect.height () * d));
 }
 
-void Matrix::invXYWH (Single & x, Single & y, Single & w, Single & h) const {
+SRect Matrix::toUser (const IRect &rect) const {
     if (a > 0.00001 && d > 0.00001) {
-        w /= a;
-        h /= d;
-        x = Single ((x - tx) / a);
-        y = Single ((y - ty) / d);
+        return SRect (
+                Single ((Single (rect.x ()) - tx) / a),
+                Single ((Single (rect.y ()) - ty) / d),
+                rect.width () / a,
+                rect.height () / d);
     } else {
         warningLog () << "Not invering " << a << ", " << d << " scale" << endl;
+        return SRect ();
     }
 }
 
@@ -97,6 +101,18 @@ Surface::~Surface() {
         cairo_surface_destroy (surface);
 }
 
+template <> void TreeNode<Surface>::appendChild (Surface *c) {
+    appendChildImpl (c);
+}
+
+template <> void TreeNode<Surface>::insertBefore (Surface *c, Surface *b) {
+    insertBeforeImpl (c, b);
+}
+
+template <> void TreeNode<Surface>::removeChild (SurfacePtr c) {
+    removeChildImpl (c);
+}
+
 void Surface::clear () {
     m_first_child = 0L;
     background_color = 0;
@@ -155,7 +171,7 @@ Surface *Surface::createSurface (NodePtr owner, const SRect & rect) {
     return surface;
 }
 
-KDE_NO_EXPORT IRect Surface::toScreen (Single x, Single y, Single w, Single h) {
+KDE_NO_EXPORT IRect Surface::toScreen (const SSize &size) {
     //FIXME: handle scroll
     Matrix matrix (0, 0, xscale, yscale);
     matrix.translate (bounds.x (), bounds.y ());
@@ -163,21 +179,19 @@ KDE_NO_EXPORT IRect Surface::toScreen (Single x, Single y, Single w, Single h) {
         matrix.transform(Matrix (0, 0, s->xscale, s->yscale));
         matrix.translate (s->bounds.x (), s->bounds.y ());
     }
-    matrix.getXYWH (x, y, w, h);
-    return IRect (x, y, w, h);
+    return matrix.toScreen (SRect (0, 0, size));
 }
 
 static void clipToScreen (Surface *s, Matrix &m, IRect &clip) {
     Surface *ps = s->parentNode ();
     if (!ps) {
-        clip = IRect (s->bounds);
+        clip = IRect (s->bounds.x (), s->bounds.y (),
+                s->bounds.width (), s->bounds.height ());
         m = Matrix (s->bounds.x (), s->bounds.y (), s->xscale, s->yscale);
     } else {
         clipToScreen (ps, m, clip);
-        SRect r = s->bounds;
-        Single x = r.x (), y = r.y (), w = r.width (), h = r.height ();
-        m.getXYWH (x, y, w, h);
-        clip = clip.intersect (IRect (x, y, w, h));
+        IRect scr = m.toScreen (s->bounds);
+        clip = clip.intersect (scr);
         Matrix m1 = m;
         m = Matrix (s->bounds.x (), s->bounds.y (), s->xscale, s->yscale);
         m.transform (m1);
@@ -187,13 +201,12 @@ static void clipToScreen (Surface *s, Matrix &m, IRect &clip) {
 }
 
 
-KDE_NO_EXPORT void Surface::repaint (const SRect &r) {
+KDE_NO_EXPORT void Surface::repaint (const SRect &rect) {
     Matrix matrix;
     IRect clip;
     clipToScreen (this, matrix, clip);
-    Single x = r.x (), y = r.y (), w = r.width (), h = r.height ();
-    matrix.getXYWH (x, y, w, h);
-    clip = clip.intersect (IRect (x, y, w, h));
+    IRect scr = matrix.toScreen (rect);
+    clip = clip.intersect (scr);
     if (!clip.isEmpty ())
         view_widget->scheduleRepaint (clip);
 }
@@ -203,6 +216,6 @@ KDE_NO_EXPORT void Surface::repaint () {
     if (ps)
         ps->repaint (bounds);
     else
-        view_widget->scheduleRepaint (IRect (bounds));
+        view_widget->scheduleRepaint (IRect (bounds.x (), bounds.y (),
+                bounds.width (), bounds.height ()));
 }
-
